@@ -271,13 +271,13 @@ use PDL::Options q/iparse/;
 use File::Temp qw/tempfile tempdir/;
 use Scalar::Util q/looks_like_number/;
 
-our $VERSION = '1.012';
+our $VERSION = '1.013';
 $VERSION =~ s/_//g;
 
 ##############################
 # Exporting
 use base 'Exporter';
-our @EXPORT = qw(pgswin line points imag cont hold release erase);
+our @EXPORT = qw(pgswin line points bins imag cont hold release erase);
 our @EXPORT_OK = (@EXPORT, qw(image plot));
 
 our $API_VERSION = '1.012'; # PGS version where that API started
@@ -1112,7 +1112,7 @@ sub oplot {
     plot(@_);
 }
 
-=head2 line, points, image, imag, cont
+=head2 line, points, bins, image, imag, cont
 
 =for usage
 
